@@ -1,10 +1,10 @@
 /* xnls.h -- NLS declarations
-   Copyright (C) 1996 Free Software Foundation, Inc.
+   Copyright (C) 1996, 2007, 2009-2010 Free Software Foundation, Inc.
    Written by Greg McGary <gkm@gnu.ai.mit.edu>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -13,8 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #ifndef _i18n_h_
@@ -26,9 +25,7 @@
 
 #if ENABLE_NLS
 # define _(Text) gettext (Text)
-# ifdef HAVE_LOCALE_H
-#  include <locale.h>
-# endif
+# include <locale.h>
 #else
 # define _(Text) Text
 #endif
