@@ -1,10 +1,11 @@
 /* mkid.c -- build an identifer database
-   Copyright (C) 1986, 1995, 1996, 1999 Free Software Foundation, Inc.
+   Copyright (C) 1986, 1995-1996, 1999, 2007-2010 Free Software Foundation,
+   Inc.
    Written by Greg McGary <gkm@gnu.ai.mit.edu>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -13,8 +14,8 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*/
 
 #include <config.h>
 #include <stdlib.h>
@@ -22,26 +23,30 @@
 #include <stdio.h>
 #include <errno.h>
 #include <getopt.h>
-#include "lstat.h"
 #include <stddef.h>
 #include <unistd.h>
-#include <pathmax.h>
 #include <string.h>
-#include <dirname.h>
-#include <alloca.h>
+#include <sys/stat.h>
 #include <limits.h>
-#include <inttostr.h>
-#include <xalloc.h>
-#include <error.h>
+
+#include "alloca.h"
+#include "argv-iter.h"
+#include "closeout.h"
+#include "dirname.h"
+#include "error.h"
+#include "inttostr.h"
+#include "pathmax.h"
+#include "progname.h"
+#include "quote.h"
+#include "quotearg.h"
+#include "xalloc.h"
 
 #include "xnls.h"
 #include "idfile.h"
-#include "hash.h"
+#include "idu-hash.h"
 #include "scanners.h"
 #include "iduglobal.h"
 
-char* dirname(char* path);
-
 struct summary
 {
   struct token **sum_tokens;
@@ -51,7 +56,6 @@ struct summary
     struct summary *u_kids[8];	/* when sum_level > 0 */
 #define sum_kids sum_u.u_kids
     struct member_file *u_files[8];	/* when sum_level == 0 */
-#define sum_files sum_u.u_files
   } sum_u;
   unsigned long sum_tokens_size;
   unsigned long sum_hits_count;
@@ -60,74 +64,81 @@ struct summary
 };
 
 void usage (void);
-static void help_me (void);
-int main (int argc, char **argv);
-int ceil_log_8 (unsigned long n);
-int ceil_log_2 (unsigned long n);
-void assert_writeable (char const *file_name);
-void scan_files (struct idhead *idhp);
-void scan_member_file (struct member_file const *member);
-void scan_member_file_1 (get_token_func_t get_token, void const *args, FILE *source_FILE);
-void report_statistics (void);
-void write_id_file (struct idhead *idhp);
-unsigned long token_hash_1 (void const *key);
-unsigned long token_hash_2 (void const *key);
-int token_hash_cmp (void const *x, void const *y);
-int token_qsort_cmp (void const *x, void const *y);
-void bump_current_hits_signature (void);
-void init_hits_signature (int i);
-void free_summary_tokens (void);
-void summarize (void);
-void init_summary (void);
-struct summary *make_sibling_summary (struct summary *summary);
-int count_vec_size (struct summary *summary, unsigned char const *tail_hits);
-int count_buf_size (struct summary *summary, unsigned char const *tail_hits);
-void assert_hits (struct summary* summary);
-void write_hits (FILE *fp, struct summary *summary, unsigned char const *tail_hits);
-void sign_token (struct token *token);
-void add_token_to_summary (struct summary *summary, struct token *token);
+static int ceil_log_8 (unsigned long n);
+static int ceil_log_2 (unsigned long n);
+static void assert_writeable (char const *file_name);
+static void scan_files (struct idhead const *idhp);
+static void scan_member_file (struct member_file const *member);
+static void scan_member_file_1 (get_token_func_t get_token,
+				void const *args, FILE *source_FILE);
+static void report_statistics (void);
+static void write_id_file (struct idhead *idhp);
+static unsigned long token_hash_1 (void const *key);
+static unsigned long token_hash_2 (void const *key);
+static int token_hash_cmp (void const *x, void const *y);
+static int token_qsort_cmp (void const *x, void const *y);
+static void bump_current_hits_signature (void);
+static void init_hits_signature (int i);
+static void free_summary_tokens (void);
+static void summarize (void);
+static void init_summary (void);
+static struct summary *make_sibling_summary (struct summary *summary);
+static int count_vec_size (struct summary *summary,
+			   unsigned char const *tail_hits);
+static int count_buf_size (struct summary *summary,
+			   unsigned char const *tail_hits);
+static void assert_hits (struct summary* summary);
+static void write_hits (FILE *fp, struct summary *summary,
+			unsigned char const *tail_hits);
+static void sign_token (struct token *token);
+static void add_token_to_summary (struct summary *summary, struct token *token);
 
-struct hash_table token_table;
+static struct hash_table token_table;
 
 /* Miscellaneous statistics */
-unsigned long input_chars;
-unsigned long name_tokens;
-unsigned long number_tokens;
-unsigned long string_tokens;
-unsigned long literal_tokens;
-unsigned long comment_tokens;
-unsigned long occurrences;
-unsigned long hits_length = 0;
-unsigned long tokens_length = 0;
-unsigned long output_length = 0;
+static unsigned long input_chars;
+static unsigned long name_tokens;
+static unsigned long number_tokens;
+static unsigned long string_tokens;
+static unsigned long literal_tokens;
+static unsigned long comment_tokens;
+static unsigned long occurrences;
+static unsigned long hits_length = 0;
+static unsigned long tokens_length = 0;
+static unsigned long output_length = 0;
 
-int verbose_flag = 0;
-int statistics_flag = 0;
+static int verbose_flag = 0;
+static int statistics_flag = 0;
 
-int file_name_count = 0;	/* # of files in database */
-int levels = 0;			/* ceil(log(8)) of file_name_count */
+static int levels = 0;			/* ceil(log(8)) of file_name_count */
 
-unsigned char *current_hits_signature;
+static unsigned char *current_hits_signature;
 #define INIT_TOKENS_SIZE(level) (1 << ((level) + 13))
-struct summary *summary_root;
-struct summary *summary_leaf;
+static struct summary *summary_root;
+static struct summary *summary_leaf;
 
-char const *program_name;
-
-char *lang_map_file_name = 0;
-int show_version = 0;
-int show_help = 0;
+static char *lang_map_file_name = 0;
+static int show_version = 0;
+static int show_help = 0;
 struct idhead idh;
-struct file_link *cw_dlink;
+static struct file_link *cw_dlink;
 
+void usage (void) __attribute__((__noreturn__));
 void
 usage (void)
 {
   fprintf (stderr, _("Try `%s --help' for more information.\n"),
 	   program_name);
-  exit (1);
+  exit (EXIT_FAILURE);
 }
 
+/* For long options that have no equivalent short option, use a
+   non-character as a pseudo short option, starting with CHAR_MAX + 1.  */
+enum
+{
+  FILES0_FROM_OPTION = CHAR_MAX +1,
+};
+
 static struct option const long_options[] =
 {
   { "file", required_argument, 0, 'f' },
@@ -142,10 +153,11 @@ static struct option const long_options[] =
   { "statistics", no_argument, 0, 's' },
   { "help", no_argument, &show_help, 1 },
   { "version", no_argument, &show_version, 1 },
-  { 0 }
+  { "files0-from", required_argument, NULL, FILES0_FROM_OPTION },
+  {NULL, 0, NULL, 0}
 };
 
-static void
+static void __attribute__((__noreturn__))
 help_me (void)
 {
   printf (_("\
@@ -160,12 +172,15 @@ Build an identifier database.\n\
   -x, --exclude=LANGS     exclude languages in LANGS\n\
   -l, --lang-option=L:OPT pass OPT as a default for language L (see below)\n\
   -m, --lang-map=MAPFILE  use MAPFILE to map file names onto source language\n\
-  -d, --default-lang=LANG make LANG the default source language\n\
+  -d, --default-lang=LANG  make LANG the default source language\n\
   -p, --prune=NAMES       exclude the named files and/or directories\n\
   -v, --verbose           report per file statistics\n\
   -s, --statistics        report statistics at end of run\n\
 \n\
-      --help              display this help and exit\n\
+      --files0-from=F     tokenize only the files specified by\n\
+                           NUL-terminated names in file F\n\
+\n\
+       --help              display this help and exit\n\
       --version           output version information and exit\n\
 \n\
 FILE may be a file name, or a directory name to recursively search.\n\
@@ -176,12 +191,12 @@ The following arguments apply to the language-specific scanners:\n\
 "));
   language_help_me ();
   printf (_("\nReport bugs to " PACKAGE_BUGREPORT "\n\n"));
-  exit (0);
+  exit (EXIT_SUCCESS);
 }
 
-void *heap_initial;
-void *heap_after_walk;
-void *heap_after_scan;
+static void *heap_initial;
+static void *heap_after_walk;
+static void *heap_after_scan;
 
 static void *get_process_heap(void)
 {
@@ -195,7 +210,11 @@ static void *get_process_heap(void)
 int
 main (int argc, char **argv)
 {
-  program_name = argv[0];
+  bool ok;
+  int nfiles;
+  char *files_from = NULL;
+
+  set_program_name (argv[0]);
   heap_initial = get_process_heap();
   idh.idh_file_name = DEFAULT_ID_FILE_NAME;
 
@@ -208,6 +227,8 @@ main (int argc, char **argv)
   textdomain (PACKAGE);
 #endif
 
+  atexit (close_stdout);
+
   for (;;)
     {
       int optc = getopt_long (argc, argv, "o:f:i:x:l:m:d:p:vVs",
@@ -250,6 +271,10 @@ main (int argc, char **argv)
 	  prune_file_names (optarg, cw_dlink);
 	  break;
 
+	case FILES0_FROM_OPTION:
+	  files_from = optarg;
+	  break;
+
 	case 'V':
 	  walker_verbose_flag = 1;
 	case 'v':
@@ -266,22 +291,40 @@ main (int argc, char **argv)
   if (show_version)
     {
       printf ("%s - %s\n", program_name, PACKAGE_VERSION);
-      exit (0);
+      exit (EXIT_SUCCESS);
     }
 
   if (show_help)
     help_me ();
 
-  argc -= optind;
-  argv += optind;
+  nfiles = argc - optind;
 
-  /* If no file or directory options exist, walk the current directory.  */
-  if (argc == 0)
+  struct argv_iterator *ai;
+  if (files_from)
     {
-      static char dot[] = ".";
-      static char *dotp = dot;
-      argc = 1;
-      argv = &dotp;
+      /* When using --files0-from=F, you may not specify any files
+	 on the command-line.  */
+      if (nfiles != 0)
+	{
+	  error (0, 0, _("extra operand %s"), quote (argv[optind]));
+	  fprintf (stderr, "%s\n",
+		   _("file operands cannot be combined with --files0-from"));
+	  usage();
+	}
+
+      if (! (strequ (files_from, "-") || freopen (files_from, "r", stdin)))
+	error (EXIT_FAILURE, errno, _("cannot open %s for reading"),
+	       quote (files_from));
+
+      ai = argv_iter_init_stream (stdin);
+    }
+  else
+    {
+      static char *cwd_only[2];
+      cwd_only[0] = bad_cast (".");
+      cwd_only[1] = NULL;
+      char **files = (optind < argc ? argv + optind : cwd_only);
+      ai = argv_iter_init_argv (files);
     }
 
   language_getopt ();
@@ -291,12 +334,73 @@ main (int argc, char **argv)
   parse_language_map (lang_map_file_name);
 
   /* Walk the file and directory names given on the command line.  */
-  while (argc--)
+  ok = true;
+  while (true)
     {
-      struct file_link *flink = parse_file_name (*argv++, cw_dlink);
-      if (flink)
-	walk_flink (flink, 0);
+      bool skip_file = false;
+      enum argv_iter_err ai_err;
+      char *file_name = argv_iter (ai, &ai_err);
+      if (ai_err == AI_ERR_EOF)
+        break;
+      if (!file_name)
+        {
+          switch (ai_err)
+            {
+            case AI_ERR_READ:
+              error (0, errno, _("%s: read error"), quote (files_from));
+              continue;
+
+            case AI_ERR_MEM:
+              xalloc_die ();
+
+            default:
+              assert (!"unexpected error code from argv_iter");
+            }
+        }
+      if (files_from && STREQ (files_from, "-") && STREQ (file_name, "-"))
+        {
+          /* Give a better diagnostic in an unusual case:
+             printf - | du --files0-from=- */
+          error (0, 0, _("when reading file names from stdin, "
+                         "no file name of %s allowed"),
+                 quote (file_name));
+          skip_file = true;
+        }
+
+      /* Report and skip any empty file names.  */
+      if (!file_name[0])
+        {
+          /* Diagnose a zero-length file name.  When it's one
+             among many, knowing the record number may help.
+             FIXME: currently print the record number only with
+             --files0-from=FILE.  Maybe do it for argv, too?  */
+          if (files_from == NULL)
+            error (0, 0, "%s", _("invalid zero-length file name"));
+          else
+            {
+              /* Using the standard `filename:line-number:' prefix here is
+                 not totally appropriate, since NUL is the separator, not NL,
+                 but it might be better than nothing.  */
+              unsigned long int file_number = argv_iter_n_args (ai);
+              error (0, 0, "%s:%lu: %s", quotearg_colon (files_from),
+                     file_number, _("invalid zero-length file name"));
+            }
+          skip_file = true;
+        }
+
+      if (skip_file)
+        ok = false;
+      else
+        {
+          struct file_link *flink = parse_file_name (file_name, cw_dlink);
+          if (flink)
+            walk_flink (flink, 0);
+	  /* FIXME: walk_flink can fail, so should return status.
+	     Then caller can continue with other arguments.  */
+        }
     }
+  argv_iter_free (ai);
+
   heap_after_walk = get_process_heap();
 
   mark_member_file_links (&idh);
@@ -333,14 +437,14 @@ main (int argc, char **argv)
 	report_statistics ();
     }
   else
-    error (0, 0, "nothing to do");
+    error (0, 0, _("nothing to do"));
 
-  exit (0);
+  exit (ok ? EXIT_SUCCESS : EXIT_FAILURE);
 }
 
 /* Return the integer ceiling of the base-8 logarithm of N.  */
 
-int
+static int
 ceil_log_8 (unsigned long n)
 {
   int log_8 = 0;
@@ -356,7 +460,7 @@ ceil_log_8 (unsigned long n)
 
 /* Return the integer ceiling of the base-2 logarithm of N.  */
 
-int
+static int
 ceil_log_2 (unsigned long n)
 {
   int log_2 = 0;
@@ -374,7 +478,7 @@ ceil_log_2 (unsigned long n)
    writable.  If it doesn't exist, make sure it can be created.  Exit
    with a diagnostic if this is not so.  */
 
-void
+static void
 assert_writeable (char const *filename)
 {
   if (access (filename, 06) < 0)
@@ -383,20 +487,20 @@ assert_writeable (char const *filename)
 	{
 	  char *dirname = dir_name (filename);
 	  if (access (dirname, 06) < 0)
-	    error (1, errno, _("can't create `%s' in `%s'"),
+	    error (EXIT_FAILURE, errno, _("can't create `%s' in `%s'"),
 		   base_name (filename), dirname);
 	  free(dirname);
 	}
       else
-	error (1, errno, _("can't modify `%s'"), filename);
+	error (EXIT_FAILURE, errno, _("can't modify `%s'"), filename);
     }
 }
 
 /* Iterate over all eligible files (the members of the set of scannable files).
    Create a tree8 to store the set of files where a token occurs.  */
 
-void
-scan_files (struct idhead *idhp)
+static void
+scan_files (struct idhead const *idhp)
 {
   struct member_file **members_0
     = (struct member_file **) hash_dump (&idhp->idh_member_file_table,
@@ -430,7 +534,7 @@ scan_files (struct idhead *idhp)
 
   for (;;)
     {
-      struct member_file *member = *members++;
+      const struct member_file *member = *members++;
       scan_member_file (member);
       if (members == end)
 	break;
@@ -445,7 +549,7 @@ scan_files (struct idhead *idhp)
 
 /* Open a file and scan it.  */
 
-void
+static void
 scan_member_file (struct member_file const *member)
 {
   struct lang_args const *lang_args = member->mf_lang_args;
@@ -488,7 +592,7 @@ scan_member_file (struct member_file const *member)
 /* Iterate over all tokens in the file, and merge the file's tree8
    signature into the token table entry.  */
 
-void
+static void
 scan_member_file_1 (get_token_func_t get_token, void const *args, FILE *source_FILE)
 {
   struct token **slot;
@@ -540,7 +644,7 @@ scan_member_file_1 (get_token_func_t get_token, void const *args, FILE *source_F
     }
 }
 
-void
+static void
 report_statistics (void)
 {
   printf (_("Name=%ld, "), name_tokens);
@@ -552,9 +656,13 @@ report_statistics (void)
   printf (_("Files=%ld, "), idh.idh_files);
   printf (_("Tokens=%ld, "), occurrences);
   printf (_("Bytes=%ld Kb, "), input_chars / 1024);
-  printf (_("Heap=%d+%d Kb, "), (heap_after_scan - heap_after_walk) / 1024,
-	  (heap_after_walk - heap_initial) / 1024);
-  printf (_("Output=%ld (%ld tok, %ld hit)\n"), output_length, tokens_length, hits_length);
+  printf (_("Heap=%llu+%llu Kb, "),
+	  (unsigned long long) ((char *) heap_after_scan
+				- (char *) heap_after_walk) / 1024,
+	  (unsigned long long) ((char *) heap_after_walk
+				- (char *) heap_initial) / 1024);
+  printf (_("Output=%ld (%ld tok, %ld hit)\n"),
+	  output_length, tokens_length, hits_length);
 
   hash_print_stats (&token_table, stdout);
   printf (_(", Freq=%ld/%ld=%.2f\n"), occurrences, token_table.ht_fill,
@@ -569,7 +677,7 @@ report_statistics (void)
    directory which you have no write access to, so you cannot create
    the ID file.)  */
 
-void
+static void
 write_id_file (struct idhead *idhp)
 {
   struct token **tokens;
@@ -582,15 +690,17 @@ write_id_file (struct idhead *idhp)
 
   if (verbose_flag)
     printf (_("Sorting tokens...\n"));
+
   assert (summary_root->sum_hits_count == token_table.ht_fill);
-  tokens = xrealloc (summary_root->sum_tokens, sizeof(struct token *) * token_table.ht_fill);
+  tokens = xnrealloc (summary_root->sum_tokens,
+		      token_table.ht_fill, sizeof *tokens);
   qsort (tokens, token_table.ht_fill, sizeof (struct token *), token_qsort_cmp);
 
   if (verbose_flag)
     printf (_("Writing `%s'...\n"), idhp->idh_file_name);
   idhp->idh_FILE = fopen (idhp->idh_file_name, "w+b");
   if (idhp->idh_FILE == NULL)
-    error (1, errno, _("can't create `%s'"), idhp->idh_file_name);
+    error (EXIT_FAILURE, errno, _("can't create `%s'"), idhp->idh_file_name);
 
   idhp->idh_magic[0] = IDH_MAGIC_0;
   idhp->idh_magic[1] = IDH_MAGIC_1;
@@ -600,14 +710,20 @@ write_id_file (struct idhead *idhp)
   /* write out the list of pathnames */
 
   fseek (idhp->idh_FILE, sizeof_idhead (), 0);
-  idhp->idh_flinks_offset = ftell (idhp->idh_FILE);
+  off_t off = ftello (idhp->idh_FILE);
+  if (UINT32_MAX < off)
+    error (EXIT_FAILURE, 0, _("internal limitation: offset of 2^32 or larger"));
+  idhp->idh_flinks_offset = off;
   serialize_file_links (idhp);
 
   /* write out the list of identifiers */
 
   putc ('\0', idhp->idh_FILE);
   putc ('\0', idhp->idh_FILE);
-  idhp->idh_tokens_offset = ftell (idhp->idh_FILE);
+  off = ftello (idhp->idh_FILE);
+  if (UINT32_MAX < off)
+    error (EXIT_FAILURE, 0, _("internal limitation: offset of 2^32 or larger"));
+  idhp->idh_tokens_offset = off;
 
   for (i = 0; i < token_table.ht_fill; i++, tokens++)
     {
@@ -651,39 +767,42 @@ write_id_file (struct idhead *idhp)
     }
   assert_hits (summary_root);
   idhp->idh_tokens = token_table.ht_fill;
-  output_length = ftell (idhp->idh_FILE);
+  off = ftello (idhp->idh_FILE);
+  if (UINT32_MAX < off)
+    error (EXIT_FAILURE, 0, _("internal limitation: offset of 2^32 or larger"));
+  output_length = off;
   idhp->idh_end_offset = output_length - 2;
   idhp->idh_buf_size = max_buf_size;
   idhp->idh_vec_size = max_vec_size;
 
   write_idhead (&idh);
-  if (fclose (idhp->idh_FILE) != 0)
-    error (1, errno, _("error closing `%s'"), idhp->idh_file_name);
+  if (ferror (idhp->idh_FILE) || fclose (idhp->idh_FILE) != 0)
+    error (EXIT_FAILURE, errno, _("error closing `%s'"), idhp->idh_file_name);
 }
 
 /* Define primary and secondary hash and comparison functions for the
    token table.  */
 
-unsigned long
+static unsigned long
 token_hash_1 (void const *key)
 {
   return_STRING_HASH_1 (TOKEN_NAME ((struct token const *) key));
 }
 
-unsigned long
+static unsigned long
 token_hash_2 (void const *key)
 {
   return_STRING_HASH_2 (TOKEN_NAME ((struct token const *) key));
 }
 
-int
+static int
 token_hash_cmp (void const *x, void const *y)
 {
   return_STRING_COMPARE (TOKEN_NAME ((struct token const *) x),
 			 TOKEN_NAME ((struct token const *) y));
 }
 
-int
+static int
 token_qsort_cmp (void const *x, void const *y)
 {
   return_STRING_COMPARE (TOKEN_NAME (*(struct token const *const *) x),
@@ -696,7 +815,7 @@ token_qsort_cmp (void const *x, void const *y)
 /* Advance the tree8 hit signature that corresponds to the file
    we are now scanning.  */
 
-void
+static void
 bump_current_hits_signature (void)
 {
   unsigned char *hits = current_hits_signature;
@@ -707,7 +826,7 @@ bump_current_hits_signature (void)
 
 /* Initialize the tree8 hit signature for the first file we scan.  */
 
-void
+static void
 init_hits_signature (int i)
 {
   unsigned char *hits = current_hits_signature;
@@ -720,7 +839,7 @@ init_hits_signature (int i)
     }
 }
 
-void
+static void
 free_summary_tokens (void)
 {
   struct summary *summary = summary_leaf;
@@ -731,7 +850,7 @@ free_summary_tokens (void)
     }
 }
 
-void
+static void
 summarize (void)
 {
   unsigned char const *hits_sig = current_hits_signature;
@@ -770,7 +889,7 @@ summarize (void)
   summary_leaf = make_sibling_summary (summary_leaf);
 }
 
-void
+static void
 init_summary (void)
 {
   unsigned long size = INIT_TOKENS_SIZE (0);
@@ -779,16 +898,17 @@ init_summary (void)
   summary_root->sum_tokens = xmalloc (sizeof(struct token *) * size);
 }
 
-struct summary *
+static struct summary *
 make_sibling_summary (struct summary *summary)
 {
   struct summary *parent = summary->sum_parent;
-  unsigned long size;
+  size_t size;
 
   if (parent == NULL)
     {
       levels++;
-      summary_root = summary->sum_parent = parent = xcalloc (1, sizeof(struct summary));
+      summary_root = summary->sum_parent = parent
+	= xcalloc (1, sizeof(struct summary));
       parent->sum_level = levels;
       parent->sum_kids[0] = summary;
       parent->sum_hits_count = summary->sum_hits_count;
@@ -802,7 +922,8 @@ make_sibling_summary (struct summary *summary)
       else
 	{
 	  parent->sum_tokens_size = size;
-	  parent->sum_tokens = xrealloc (summary->sum_tokens, sizeof(struct token *) * size);
+	  parent->sum_tokens = xnrealloc (summary->sum_tokens, size,
+					  sizeof *summary->sum_tokens);
 	}
       summary->sum_tokens = 0;
     }
@@ -818,7 +939,7 @@ make_sibling_summary (struct summary *summary)
   return summary;
 }
 
-int
+static int
 count_vec_size (struct summary *summary, unsigned char const *tail_hits)
 {
   struct summary **kids;
@@ -842,7 +963,7 @@ count_vec_size (struct summary *summary, unsigned char const *tail_hits)
     }
 }
 
-int
+static int
 count_buf_size (struct summary *summary, unsigned char const *tail_hits)
 {
   struct summary **kids;
@@ -863,7 +984,7 @@ count_buf_size (struct summary *summary, unsigned char const *tail_hits)
     }
 }
 
-void
+static void
 assert_hits (struct summary* summary)
 {
   struct summary **kids = summary->sum_kids;
@@ -878,7 +999,7 @@ assert_hits (struct summary* summary)
     assert_hits (*kids++);
 }
 
-void
+static void
 write_hits (FILE *fp, struct summary *summary, unsigned char const *tail_hits)
 {
   struct summary **kids;
@@ -898,7 +1019,7 @@ write_hits (FILE *fp, struct summary *summary, unsigned char const *tail_hits)
     }
 }
 
-void
+static void
 sign_token (struct token *token)
 {
   unsigned char *tok_hits = TOKEN_HITS (token);
@@ -927,15 +1048,15 @@ sign_token (struct token *token)
     }
 }
 
-void
+static void
 add_token_to_summary (struct summary *summary, struct token *token)
 {
-  unsigned long size = summary->sum_tokens_size;
+  size_t size = summary->sum_tokens_size;
 
   if (summary->sum_hits_count >= size)
     {
-      size *= 2;
-      summary->sum_tokens = xrealloc (summary->sum_tokens, sizeof(struct token *) * size);
+      summary->sum_tokens = x2nrealloc (summary->sum_tokens, &size,
+					sizeof *summary->sum_tokens);
       summary->sum_tokens_size = size;
     }
   summary->sum_tokens[summary->sum_hits_count++] = token;
